@@ -33,7 +33,7 @@ class Profile(Enum):
 class MetaExtractor:
     """
     Extract a minimal, investigation or full slice of metadata for the
-    variables referenced in df_QW['cluster_descripcion'].
+    variables referenced in df_QW['cluster_description'].
 
     *Synonyms and suffix stripping are fully inferred from the metadata.*
     """
@@ -135,13 +135,11 @@ class MetaExtractor:
     @staticmethod
     def _extract_tokens(series: pd.Series) -> Set[str]:
         """
-        Extrae de una Serie de pandas todos los nombres de variable que aparezcan:
-        • En comparaciones con operadores (<=, >=, <, >, ==, !=), incluyendo
-            números en notación científica y con guiones bajos como separadores
-            (p. ej. 1_000_000, 3.2e-4).
-        • En tokens con formato prefijo__nombre (p. ej. cat__score).
+        Extract all variable names appearing in a pandas Series:
+        • In comparisons with operators (<=, >=, <, >, ==, !=), including numbers in scientific notation and with underscores as separators (e.g. 1_000_000, 3.2e-4).
+        • In tokens formatted as prefix__name (e.g. cat__score).
 
-        Devuelve un conjunto único de nombres de variable.
+        Returns a unique set of variable names.
         """
         # 1) convertimos la serie a un solo string
         text = " ".join(series.dropna().astype(str))
@@ -150,9 +148,9 @@ class MetaExtractor:
         var_re = r"[A-Za-z_][A-Za-z0-9_\.\[\]]*"       # variables Python-like
         num_re = r"""
             [-+]?                                     # signo opcional
-            \d[\d_]*                                  # parte entera (1 ó 1_000_000)
-            (?:\.\d[\d_]*)?                           # parte decimal opcional
-            (?:[eE][-+]?\d+)?                         # notación científica opcional
+            \d[\d_]*                                  # integer part (1 or 1_000_000)
+            (?:\.\d[\d_]*)?                           # optional decimal part
+            (?:[eE][-+]?\d+)?                         # optional scientific notation
         """
         op_re  = r"(?:<=|>=|<|>|==|!=)"                # operadores permitidos
 
@@ -170,7 +168,7 @@ class MetaExtractor:
         for m in compare_re.finditer(text):
             for side in ("left", "right"):
                 lex = m.group(side)
-                # si lex es variable (no numérico), la añadimos
+                # if lex is a variable (non-numeric) add it
                 if re.fullmatch(var_re, lex) and not re.fullmatch(num_re, lex):
                     tokens.add(lex)
 
@@ -197,7 +195,7 @@ class MetaExtractor:
         hit, score = _fuzzy_extract(canon_lc, [v.lower() for v in self.meta.index])
         return hit if hit and score >= 65 else None
 
-    # ───────────────────────── API pública ─────────────────────────
+    # ───────────────────────── Public API ─────────────────────────
     def extract(
         self,
         df_QW: pd.DataFrame,
@@ -206,11 +204,11 @@ class MetaExtractor:
         cols: List[str] | None = None
     ) -> pd.DataFrame:
 
-        if "cluster_descripcion" not in df_QW.columns:
-            raise KeyError("df_QW must contain 'cluster_descripcion' column")
+        if "cluster_description" not in df_QW.columns:
+            raise KeyError("df_QW must contain 'cluster_description' column")
 
-        # tokens = self._tokens(df_QW["cluster_descripcion"])
-        tokens = self._extract_tokens(df_QW["cluster_descripcion"])
+        # tokens = self._tokens(df_QW["cluster_description"])
+        tokens = self._extract_tokens(df_QW["cluster_description"])
         mapping = {t: self._map_to_var(self._canon(t)) for t in tokens}
         valid   = {tok: var for tok, var in mapping.items() if var in list(self.meta.index)}
 
@@ -237,111 +235,187 @@ class MetaExtractor:
 # 1. UTILITARIOS DE PARSEO
 # ------------------------------------------------------------------ #
 def parse_rule_string(rule_str: str) -> list[str]:
-    """Devuelve lista de condiciones limpias quitando AND y espacios extra."""
+    """Split a rule string into individual conditions.
+
+    Parameters
+    ----------
+    rule_str : str
+        Text with conditions joined by ``AND``. Parentheses and extra
+        whitespace are ignored.
+
+    Returns
+    -------
+    list[str]
+        Clean list of condition strings. ``[]`` is returned for NaN inputs.
+    """
     if pd.isna(rule_str):
         return []
-    # Normaliza espacios y quita paréntesis (si hubiera)
-    parts = [re.sub(r'\s+', ' ', p.strip('() ')) for p in rule_str.split('AND')]
-    return [p for p in parts if p]                 # sin strings vacíos
+    # Normalize spaces and remove parentheses if present
+    parts = [re.sub(r"\s+", " ", p.strip("() ")) for p in rule_str.split("AND")]
+    return [p for p in parts if p]  # no empty strings
 
 def token_from_condition(cond: str) -> str | None:
+    """Extract the variable token within a condition string.
+
+    Parameters
+    ----------
+    cond : str
+        Condition such as ``"-3.2 <= num__age <= 1.5"``.
+
+    Returns
+    -------
+    str or None
+        Token found in the condition (e.g. ``"num__age"``) or ``None`` if no
+        token can be identified.
     """
-    Extrae el token de la variable dentro de la condición.
-    Ej.: '-3.2 <= num__age <= 1.5' → 'num__age'
-    """
-    # busca la palabra con '__' (primera preferencia)
+    # search for word containing '__' first
     for word in cond.split():
         if '__' in word:
             return word
-    # respaldo: primera palabra alfabética
+    # fallback: first word containing alphabetic characters
     for word in cond.split():
         if re.search(r'[A-Za-z]', word):
             return word
     return None
 
 def conditions_to_tokens(conds: list[str]) -> set[str]:
+    """Extract tokens from a list of conditions.
+
+    Parameters
+    ----------
+    conds : list[str]
+        List of condition strings.
+
+    Returns
+    -------
+    set[str]
+        Set of unique tokens present in ``conds``.
+    """
     return {token_from_condition(c) for c in conds if token_from_condition(c)}
 
 # ------------------------------------------------------------------ #
 # 2. GENERADOR DE EXPERIMENTOS PARA UN SOLO Df2
 # ------------------------------------------------------------------ #
 
-def experiments_from_df2(df2: pd.DataFrame,
-                         meta: pd.DataFrame) -> pd.DataFrame:
+def experiments_from_df2(
+    df2: pd.DataFrame,
+    meta: pd.DataFrame,
+    df_data: pd.DataFrame | None = None,
+    var_obj: str = "target",
+) -> pd.DataFrame:
+    """Generate pairwise cluster comparisons for a single Df2.
+
+    Parameters
+    ----------
+    df2 : pd.DataFrame
+        Table with one row per cluster including ``cluster_description``,
+        ``cluster_ef_sample`` and ``cluster_n_sample`` columns.
+    meta : pd.DataFrame
+        Metadata indexed by ``rule_token`` providing actionability metrics.
+    df_data : pd.DataFrame, optional
+        Raw dataset used to compute cluster descriptions. If provided, the
+        effectiveness and number of observations for the intersection between
+        clusters is computed by applying the shared rules to ``df_data``.
+    var_obj : str, default "target"
+        Name of the target column inside ``df_data`` used to estimate
+        effectiveness.
+
+    Returns
+    -------
+    pd.DataFrame
+        Each row contains the comparison between two clusters along with the
+        exclusive variables, intersection statistics and a score penalizing
+        difficult actions and rewarding overlap.
     """
-    Devuelve UNA fila por par de clústeres con:
-      · variables_low  : lista de tokens exclusivos del clúster de menor efectividad
-      · variables_high : lista de tokens exclusivos del clúster de mayor efectividad
-      · difficulty_low : máx(actionability.increase_difficulty) en variables_low
-      · difficulty_high: máx(actionability.decrease_difficulty) en variables_high
-      · n_intersection, n_only_low, n_only_high (conteos)
-      · score (penaliza dificultad_low, exclusivas y premia intersección)
-    """
-    # --- tabla de acción -----------------------------
+    # --- action table -----------------------------
     meta_idx = meta.set_index('rule_token')
 
     recs = []
     for (_, row_a), (_, row_b) in itertools.combinations(df2.iterrows(), 2):
 
-        conds_a = set(parse_rule_string(row_a['cluster_descripcion']))
-        conds_b = set(parse_rule_string(row_b['cluster_descripcion']))
+        conds_a = set(parse_rule_string(row_a['cluster_description']))
+        conds_b = set(parse_rule_string(row_b['cluster_description']))
 
         inters     = sorted(conds_a & conds_b)
         only_a     = sorted(conds_a - conds_b)
         only_b     = sorted(conds_b - conds_a)
 
-        # ¿quién es el clúster "low" (menor efectividad)?
-        delta_ef   = row_a['cluster_ef_sample'] - row_b['cluster_ef_sample']
-        low_row, high_row  = (row_a, row_b) if delta_ef < 0 else (row_b, row_a)
-        only_low, only_high = (only_a, only_b) if delta_ef < 0 else (only_b, only_a)
+        # -------------------- intersection stats -----------------------
+        inter_ef = None
+        inter_n = None
+        if df_data is not None and inters:
+            def _apply_conditions(data, conds):
+                for cond in conds:
+                    match = re.match(
+                        r"\s*([-\d\.eE]+)\s*<=\s*([A-Za-z_][A-Za-z0-9_]*)\s*<=\s*([-\d\.eE]+)",
+                        str(cond),
+                    )
+                    if match and match.group(2) in data.columns:
+                        low = float(match.group(1))
+                        high = float(match.group(3))
+                        var = match.group(2)
+                        data = data[(data[var] >= low) & (data[var] <= high)]
+                return data
 
-        # -------------------- listas de tokens exclusivas ------------------
+            df_inter = _apply_conditions(df_data.copy(), inters)
+            inter_n = int(df_inter.shape[0])
+            if var_obj in df_inter.columns and inter_n > 0:
+                inter_ef = float(df_inter[var_obj].mean())
+
+        # Determine which cluster has lower effectiveness
+        delta_ef   = row_a['cluster_ef_sample'] - row_b['cluster_ef_sample']
+        row_a_subset, row_b_subset = (row_a, row_b) if delta_ef < 0 else (row_b, row_a)
+        only_subset_a, only_subset_b = (only_a, only_b) if delta_ef < 0 else (only_b, only_a)
+
+        # -------------------- lists of exclusive tokens ------------------
         def to_tokens(cond_list):
             return sorted({token_from_condition(c) for c in cond_list
                            if token_from_condition(c) is not None})
 
-        vars_low  = to_tokens(only_low)
-        vars_high = to_tokens(only_high)
+        vars_a = to_tokens(only_subset_a)
+        vars_b = to_tokens(only_subset_b)
 
-        # -------------------- máximos de dificultad ------------------------
+        # -------------------- maximum difficulty ------------------------
         def max_difficulty(tokens, col):
             vals = [meta_idx.at[t, col] for t in tokens
                     if t in meta_idx.index and pd.notna(meta_idx.at[t, col])]
             return max(vals) if vals else 10        # castigo alto si faltan datos
 
-        difficulty_low  = max_difficulty(vars_low,  'actionability.increase_difficulty')
-        difficulty_high = max_difficulty(vars_high, 'actionability.decrease_difficulty')
+        difficulty_a = max_difficulty(vars_a, 'actionability.increase_difficulty')
+        difficulty_b = max_difficulty(vars_b, 'actionability.decrease_difficulty')
 
         # -------------------- score --------------------
-        n_tot   = (low_row.get('cluster_n_sample', 0) or 0) + \
-                  (high_row.get('cluster_n_sample', 0) or 0)
+        n_tot   = (row_a_subset.get('cluster_n_sample', 0) or 0) + \
+                  (row_b_subset.get('cluster_n_sample', 0) or 0)
         weight  = max(n_tot ** 0.5, 1)
 
         n_inter = len(inters)
-        n_only_low  = len(vars_low)
-        n_only_high = len(vars_high)
+        n_only_a = len(vars_a)
+        n_only_b = len(vars_b)
 
         score = (abs(delta_ef) * weight * (1 + n_inter) /
-                 (difficulty_low + 0.5) /
-                 (1 + n_only_low + n_only_high))
+                 (difficulty_a + 0.5) /
+                 (1 + n_only_a + n_only_b))
 
         recs.append({
-            'cluster_low'        : int(low_row['cluster']),
-            'cluster_high'       : int(high_row['cluster']),
-            'cluster_ef_low'     : row_a['cluster_ef_sample'],
-            'cluster_ef_high'    : row_b['cluster_ef_sample'],
+            'cluster_a'          : int(row_a_subset['cluster']),
+            'cluster_b'          : int(row_b_subset['cluster']),
+            'cluster_ef_a'       : row_a['cluster_ef_sample'],
+            'cluster_ef_b'       : row_b['cluster_ef_sample'],
             'delta_ef'           : abs(delta_ef),
             'avg_n'              : n_tot / 2,
-            'variables_low'      : vars_low,
-            'variables_high'     : vars_high,
-            'difficulty_low'     : difficulty_low,
-            'difficulty_high'    : difficulty_high,
+            'variables_a'        : vars_a,
+            'variables_b'        : vars_b,
+            'difficulty_a'       : difficulty_a,
+            'difficulty_b'       : difficulty_b,
             'n_intersection'     : n_inter,
-            'n_only_low'         : n_only_low,
-            'n_only_high'        : n_only_high,
+            'n_only_a'           : n_only_a,
+            'n_only_b'           : n_only_b,
             'intersection'       : inters,
-            'only_cluster_low'   : only_low,
-            'only_cluster_high'  : only_high,
+            'intersection_ef_sample': inter_ef,
+            'intersection_n_sample' : inter_n,
+            'only_cluster_a'     : only_subset_a,
+            'only_cluster_b'     : only_subset_b,
             'score'              : score,
         })
 
@@ -352,24 +426,54 @@ def experiments_from_df2(df2: pd.DataFrame,
 # ------------------------------------------------------------------ #
 # 3. PIPELINE GENERAL PARA «n» Df2
 # ------------------------------------------------------------------ #
-def run_experiments(mx, df2_dict: dict[str, pd.DataFrame]) -> pd.DataFrame:
-    """
-    Genera y consolida hipótesis para un diccionario de Df2.
+def run_experiments(
+    mx,
+    df2_dict: dict[str, pd.DataFrame],
+    data_dict: dict[str, pd.DataFrame] | None = None,
+) -> pd.DataFrame:
+    """Generate and consolidate hypotheses for multiple datasets.
+
+    Parameters
+    ----------
+    mx : MetaExtractor
+        Instance used to extract metadata from ``cluster_description`` fields.
+    df2_dict : dict[str, pd.DataFrame]
+        Mapping of dataset name to its corresponding Df2 table.
+    data_dict : dict[str, pd.DataFrame], optional
+        Optional mapping of dataset name to the raw data used to create each
+        Df2. When provided, intersection effectiveness and observation counts
+        are computed using these DataFrames.
+
+    Returns
+    -------
+    pd.DataFrame
+        Concatenated hypotheses ordered by score. If no hypotheses are
+        generated an empty DataFrame with the expected columns is returned.
     """
     all_hypotheses = []
 
     for name, df2 in df2_dict.items():
         df1  = mx.extract(df2)
-        hypo = experiments_from_df2(df2, df1)
+        df_raw = data_dict.get(name) if data_dict else None
+        hypo = experiments_from_df2(df2, df1, df_raw, var_obj=getattr(mx, "var_obj", "target"))
 
         if not hypo.empty:
             hypo['dataset'] = name
+            hypo['variables_intersection'] = [
+                sorted({token_from_condition(c) for c in conds
+                        if token_from_condition(c) is not None})
+                for conds in hypo['intersection']
+            ]
             all_hypotheses.append(hypo)
 
     if not all_hypotheses:
-        cols = ['dataset', 'cluster_low', 'cluster_high',
-                'variable', 'condition', 'delta_ef',
-                'avg_n', 'difficulty', 'score']
+        cols = ['dataset', 'cluster_a', 'cluster_b',
+                'cluster_ef_a', 'cluster_ef_b', 'delta_ef', 'avg_n',
+                'variables_a', 'variables_b', 'variables_intersection',
+                'difficulty_a', 'difficulty_b', 'n_intersection',
+                'n_only_a', 'n_only_b', 'intersection',
+                'intersection_ef_sample', 'intersection_n_sample',
+                'only_cluster_a', 'only_cluster_b', 'score']
         return pd.DataFrame(columns=cols)
 
     return (pd.concat(all_hypotheses, ignore_index=True)
